/**
 * @license
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _CachingPackageJsonReader_cache;
import { AsyncCache } from './async-cache.js';
import { PackageJson } from './package-json.js';
import * as pathlib from 'path';
import * as fs from 'fs/promises';
import { parseTree } from './ast.js';
export const astKey = Symbol('ast');
/**
 * Reads package.json files and caches them.
 */
export class CachingPackageJsonReader {
    constructor() {
        _CachingPackageJsonReader_cache.set(this, new AsyncCache());
    }
    async read(packageDir) {
        return __classPrivateFieldGet(this, _CachingPackageJsonReader_cache, "f").getOrCompute(packageDir, async () => {
            const path = pathlib.resolve(packageDir, 'package.json');
            let contents;
            try {
                contents = await fs.readFile(path, 'utf8');
            }
            catch (error) {
                if (error.code === 'ENOENT') {
                    return {
                        ok: false,
                        error: {
                            type: 'failure',
                            reason: 'missing-package-json',
                            script: { packageDir },
                        },
                    };
                }
                throw error;
            }
            const astResult = parseTree(path, contents);
            if (!astResult.ok) {
                return astResult;
            }
            const packageJsonFile = new PackageJson({ contents, path }, astResult.value);
            return { ok: true, value: packageJsonFile };
        });
    }
    async *getFailures() {
        const values = await Promise.all([...__classPrivateFieldGet(this, _CachingPackageJsonReader_cache, "f").values]);
        for (const result of values) {
            if (!result.ok) {
                yield result.error;
                continue;
            }
            const packageJson = result.value;
            yield* packageJson.failures;
        }
    }
}
_CachingPackageJsonReader_cache = new WeakMap();
//# sourceMappingURL=package-json-reader.js.map